/* autoprefix.h
 *  Copyright: 2004 The Perl Foundation.  All Rights Reserved.
 *  CVS Info
 *     $Id: autoprefix.h 6520 2004-08-09 21:53:50Z dan $
 *  Overview:
 *     provide Parrot_ prefixes for functions that don't have them
 *  Data Structure and Algorithms:
 *  History:
 *  Notes:
 *  References:
 */

#if !defined(PARROT_AUTOPREFIX_H_GUARD)
#define PARROT_AUTOPREFIX_H_GUARD

#if !defined HAS_LINKER_EXPORT_RESTRICTION
/* The prefixed symbols go here */

#endif

#endif   /* PARROT_AUTOPREFIX_H_GUARD */

/*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
*/
