# $Id: 013_ifdef.t 6200 2004-06-03 07:35:43Z leo $

use strict;

use Parrot::Test tests => 1;

{
  language_output_is( 'm4', <<'CODE', <<'OUT', 'simple define' );
define(`foo', `Hello World')
ifdef(`foo', `f o o is defined', `f o o is not defined')
ifdef(`foo2', `f o o 2 is defined', `f o o 2 is not defined')
CODE

f o o is defined
f o o 2 is not defined
OUT
}
