# $Id: 007_empty_line.t 6200 2004-06-03 07:35:43Z leo $

use strict;

use Parrot::Test tests => 1;

language_output_is( 'm4', <<'CODE', <<'OUT', 'hello' );

Hello World



Hallo Welt
CODE

Hello World



Hallo Welt
OUT
