/*
Copyright: 2001-2003 The Perl Foundation.  All Rights Reserved.
$Id: parrot.c 5158 2004-01-27 23:08:08Z mikescott $

=head1 NAME

src/parrot.c - 

=head1 DESCRIPTION

THIS FILE IS NOT USED.

=cut

*/

#include "parrot/parrot.h"

/*

=head1 HISTORY

"It was originally supposed to serve the role of C<test_main.c>, but that
didn't ever happen for some reason. Ask Simon if you really want to
know." -- Brent Dax

=cut

*/

/*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 4
 * indent-tabs-mode: nil 
 * End:
 *
 * vim: expandtab shiftwidth=4:
*/
