a+?
INPUT:
x
OUTPUT:
Match failed
INPUT:
a
OUTPUT:
Match found
0: 0..0
INPUT:
aaaaax
OUTPUT:
Match found
0: 0..4
INPUT:
aaaaa
OUTPUT:
Match found
0: 0..4
INPUT:
yaaaaaay!!!
OUTPUT:
Match found
0: 1..6
