# $Id: 015___file__.t 7041 2004-11-17 09:24:49Z leo $

use strict;

use Parrot::Test tests => 1;

{
  language_output_is( 'm4', <<'CODE', <<'OUT', '__line__' );
Currently the file '__file__' is being processed.
CODE
Currently the file 'languages/m4/t/builtins/015___file___1.m4' is being processed.
OUT
}
